-- @Name State_Game
-- @Type parent
global gText, gHud

property pGameBuffer
property pObjects
property pInitiated
property pLastFrameMS
property pButtons
property pFlowerSpeed
property pRules
property pLevel
property pShowRules
property pShowResult
property pLevelName

property ancestor

on new me
  global gGame
  
  me.ancestor = script("StateInterface").new()
  
  pGameBuffer = member("GameBuffer").image
  pInitiated = false
  pButtons = [#use:false, #useKeyUp: false, #useKeyDown: false ,#plant:false, #plantKeyUp: false, #plantKeyDown: false]
  
  gGame = me
  
  me.Reset()
  
  return me
end

on Start me 
  
  go to "Game"
  
  pGameBuffer.fill(0,0,320,240,[#color: rgb(0,0,0)])
  
  if not pInitiated then
    me.Init()
  end if
  
  gHUD.pDirty = true
  
  --Reset lastframe time, so the diff will be 0, nothing will move the first frame.
  pLastFrameMS = the milliseconds
  
  if not pShowRules and not pShowResult then
    sound(1).fadeOut(1000)      
    sound(2).fadeIn(1000)
  end if
  
end

on Stop me
  --put "Stop "& me
end

on KeyUp me
  global gPlayer  
  --put the key && the keycode  
  if the keycode = 53 Then  -- ESC
    
    if not pShowRules and not pShowResult then
      sound(1).fadeIn(1000)      
      sound(2).fadeOut(1000)
    end if
    
    SetState("MainMenu")
  end if
  
  -- "+ 78"
  -- "+ 24"
  -- "- 70"
  -- "- 27"
  if the shiftdown then
    if the keycode = 78 or the keycode = 24 then
      CreateLevel(pLevel + 1)
    else
      if the keycode = 70 or the keycode = 24 then
        CreateLevel( max(1,pLevel - 1) )
      end if
    end if
    
  end if
  
end

on AddObject me, aObject
  
  if objectP( aObject ) then
    pObjects.append( aObject )
  end if
  
end

on EnterFrame me
  
  if not pShowRules and not pShowResult then
    
    vDiffMS = the milliseconds - pLastFrameMS
    pLastFrameMS = the milliseconds 
    
    --Clear dead objects
    repeat with i = pObjects.count down to 1
      if pObjects[i].pRemoveMe then
        pObjects[i].terminate()
        pObjects.deleteAt(i)
      end if
    end repeat
    
    
    if pRules.CheckWin() or not pRules.CheckSomethingToDo() then
      -- Level Done.
      
      sound(1).fadeIn(1000)      
      sound(2).fadeOut(1000)
      
      pShowResult = true
      --CreateLevel( me.pLevel + 1 )
      
      return
      
    end if
    
  end if
  
  
  me.HandleInput()
  
  --gamelogic 
  if not pShowRules and not pShowResult then
    Call #EnterFrame, pObjects, vDiffMS  
  end if
  
  --render  
  Call #render, pObjects, pGameBuffer, point(0,0), [#ink:#backgroundtransparent]
  
  if pShowRules then
    pRules.Render( pGameBuffer, point(160,120) - point(150,160)/2 )
  end if
  
  if pShowResult then
    pRules.RenderResult( pGameBuffer, point(160,120) - point(150,160)/2 )
  end if
  
  gHUD.render( pGameBuffer, point(0,0) )
  
  --gText.Render(pGameBuffer,point(0,0), "THE FARMER BY DRZOOL")
  
  member("GameBuffer").image = pGameBuffer
  
end

on HandleInput me
  global gPlayer
  
  
  Arrows = point(0,0)
  
  if keypressed(123) then Arrows.loch = - 1
  if keypressed(124) then Arrows.loch = Arrows.loch + 1
  if keypressed(125) then Arrows.locv = 1
  if keypressed(126) then Arrows.locv = Arrows.locv - 1
  
  --BUTTONS
  
  --pButtons.plantKeyDown = false 
  --pButtons.plantKeyUp = false 
  
  pButtons.useKeyDown = false 
  pButtons.useKeyUp = false  
  
  --PLANT CTRL
  --  if the optiondown then 
  --    if pButtons.plant = false then pButtons.plantKeyDown = true   
  --    
  --    pButtons.plant = true
  --    
  --  else 
  --    if pButtons.plant = true then pButtons.plantKeyUp = true 
  --    
  --    pButtons.plant = false
  --  end if
  
  --USE - SPACE
  if keypressed( 49 ) then 
    if pButtons.use = false then pButtons.useKeyDown = true   
    
    pButtons.use = true
    
  else 
    if pButtons.use = true then pButtons.useKeyUp = true 
    
    pButtons.use = false
  end if
  
  if pShowRules then
    
    if pButtons.useKeyDown then      
      pShowRules = false
      --Reset lastframe time, so the diff will be 0, nothing will move the first frame.
      pLastFrameMS = the milliseconds      
      
    end if
    
  else
    if pShowResult then      
      if pButtons.useKeyDown then      
        pShowResult = false
        
        if pRules.CheckWin() then          
          CreateLevel( pLevel + 1 )
        else          
          CreateLevel( pLevel  )
        end if
        
      end if
      
    else
      
      if objectP(gPlayer) then gPlayer.Input( Arrows, pButtons )
      
      
    end if
  end if
  
end

on Init me
  global gPlayer  
  
  CreateLevel(1)
  
  pInitiated = true
end

on GetFlowerSpeed me
  return pFlowerSpeed
end

on terminate me
  call #terminate, pObjects
  gAnim = void  
end

on Reset me
  
  --Erase all prev objects
  if listp(pFlowerSpeed) then
    Call #terminate, pFlowerSpeed
  end if
  
  pObjects = []
  
  pFlowerSpeed = 1800  
  
  pRules = script("Rules").new()
  
  pShowRules = true
  
end

