-- @Name player
-- @Type parent
Global gHUD
Global gGame
Global gSoundFx

property ancestor
property pOnGround
property pVel --Velocity
property pJumpPower
property pSpeedLimit
property pFriction
property pAirFriction
property pGroundAcc
property pAirAcc
property pSeeds
property pFlowerValue
property pFlowers
property pMoney
property pFertelizer


on new me
  
  me.ancestor = script("anim").new() 
  me.pType = #player
  
  me.pLoc = point(160,120)
  me.SetAnim("FarmerIdle")
  
  pOnGround = false
  
  pVel = point(.0, .0)
  
  pJumpPower = 8.0
  pSpeedLimit = 4.5
  pFriction = 0.7
  pAirFriction = 0.8
  pGroundAcc = 1.0
  pAirAcc = 5.0
  pSeeds = 7
  pFlowerValue = 0
  pFlowers = 0
  pMoney = 0
  pFertelizer = 0
  
  gHUD.pDirty = true
  
  return me
end

on Input (me, Arrows, Buttons)
  --Control the player
  
  if pOnGround then
    if Arrows.loch = 0 then    
      me.SetAnim("FarmerIdle")
      
    else
      me.SetAnim("FarmerWalk")      
      
    end if 
  end if
  
  if Arrows.loch < 0 then
    me.pFlipH = false
  else if Arrows.loch > 0 then
    me.pFlipH = true      
  end if
  
  if pOnGround then
    if Arrows.locv <= 0 then --Dont move when crouching
      me.pVel.loch = max( -pSpeedLimit, min( pSpeedLimit, me.pVel.loch + Arrows.loch / pGroundAcc))
    else
      me.pVel.loch = 0
      me.SetAnim("FarmerCrouch")
    end if
    
    if Arrows.locv < 0 then
      --Jump
      pVel.locv = pVel.locv - pJumpPower
      pOnGround = false
      me.SetAnim("FarmerJump")
      
    end if
    
  else
    --In air
    me.pVel.loch = max( -pSpeedLimit, min( pSpeedLimit, me.pVel.loch + Arrows.loch / pGroundAcc))
    
  end if
  
  if Arrows.locv > 0 then
    --Down and space, plant
    if Buttons.useKeyDown then me.Plant()
    
  else
    --all else use
    if Buttons.useKeyDown then me.Use()
    
  end if
  
end

on Plant me
  global gGame
  
  if pOnGround then  
    
    if pSeeds > 0 then
      
      if me.CanPlantHere() then
        pSeeds = pSeeds - 1
        
        vFlower = script("Flower").new()
        
        if pFertelizer > 0 then
          pFertelizer = pFertelizer - 1
          vFlower.Fertelize()
        end if
        vFlower.pLoc = me.pLoc    
        
        gGame.AddObject( vFlower )
        
        gHUD.pDirty = true
                
        gSoundFx.Play( member("sPlant") )        
        
      else
        
        gSoundFx.Play( member("sNotRoom") )        
        
      end if
    else
      
      --Play no seeds sound
    end if
    
  end if
  
end

on CanPlantHere me
  --return if this place can be planted
  
  --Find stuff in range
  vInRange = me.GetObjectsInRange(-10,10)
  
  repeat with vObj in vInRange    
    if vObj.pType = #flower then return false
    
  end repeat
  
  return true
end

on Use me
  global gGame
  
  if pOnGround then    
    --Find objects infront of player,
    --Use them according to what they are
    
    --Find stuff in range
    vInRange = me.GetObjectsInRange(-5,20)
    
    
    --Use objects
    repeat with vObj in vInRange
      
      case vObj.pType of
        #flower:
          vResult = vObj.Mow()
          
          case vResult.type of
            #seeds:
              me.AddSeeds( vResult.value )
              
              if vResult.value = 2 then
                gSoundFx.Play( member("sSeeds") )
              else
                gSoundFx.Play( member("sLotofSeeds") )                
              end if
              
            #flower:
              me.AddFlower( vResult.value )              
              
              if vResult.value = 1 then
                gSoundFx.Play( member("sFlower") )
              else
                gSoundFx.Play( member("sNiceFlower") )                
              end if
              
            #fertelizer:
              me.AddFertelizer( vResult.value )
              
              if vResult.value = 1 then
                gSoundFx.Play( member("sFertelizer") )
              else
                gSoundFx.Play( member("sGreatFertelizer") )                
              end if
          end case
          
          me.SetAnim("FarmerCrouch")
          
        #stand: me.SellFlowers()    
          
          
      end case
      
    end repeat
    
    
    vInRange = void
  end if
  
end

on GetObjectsInRange ( me, vRange, vFarRange)
  global gGame
  
  --Find stuff in range
  vInRange  = []
  
  repeat with vObj in gGame.pObjects
    if vObj = me then next repeat
    
    if me.pFlipH then
      if vObj.pLoc.locH > me.pLoc.loch+vRange and vObj.pLoc.locH < me.pLoc.loch+vFarRange then
        
        vInRange.append(vObj)
        
      end if
      
    else
      if vObj.pLoc.locH < me.pLoc.loch-vRange and vObj.pLoc.locH > me.pLoc.loch-vFarRange then
        
        vInRange.append(vObj)
        
      end if
    end if
    
  end repeat
  
  return vInRange
end

on AddFertelizer me, aAmount
  me.pFertelizer = me.pFertelizer + aAmount
  
  gHUD.pDirty = true  
  
  gGame.AddObject( script("Bling").new(#fertalizer, me.pLoc) )
  
  
end

on SteelSeed me
  if me.pSeeds < 1 then return
  
  me.pSeeds = me.pSeeds - 1  
  gHUD.pDirty = true  
  
  gSoundFx.Play( member("ouch") )
  
end

on AddSeeds me, aAmount
  me.pSeeds = me.pSeeds + aAmount
  
  gHUD.pDirty = true
  
  gGame.AddObject( script("Bling").new(#seed, me.pLoc) )
end

on AddFlower me, aValue
  me.pFlowerValue = me.pFlowerValue + aValue  
  me.pFlowers = me.pFlowers + 1
  
  gHUD.pDirty = true  
  
  gGame.AddObject( script("Bling").new(#flower, me.pLoc) )
end

on SellFlowers me
  
  if pFlowerValue> 0 then
    me.pMoney = me.pMoney + pFlowerValue
    me.pFlowers = 0
    me.pFlowerValue = 0
    
    gHUD.pDirty = true
    
    gGame.AddObject( script("Bling").new(#money, me.pLoc) )    
    
    if pFlowerValue = 1 then
      gSoundFx.Play( member("sCoin") )
    else
      gSoundFx.Play( member("sCoins") )                
    end if
    
  else
    --Play no flower sound
  end if
end

on EnterFrame me, aTime
  --Update mechanics
  
  if pOnGround then
    --friction    
    me.pVel.loch = me.pVel.loch * pFriction
    
  else
    --friction    
    me.pVel.loch = me.pVel.loch * pAirFriction
    
    --gravity
    me.pVel.locv = me.pVel.locv + (0.98)
    
  end if
  
  
  --Movement
  me.pLoc = me.pLoc + (me.pVel)
  
  if pOnGround then
    
  else
    
    --Check land on bug
    if me.pLoc.locv > 184 then
      
      if me.pType = #player then
        me.SquashBugsBelow()
      end if
      
    end if
    
    --floor
    if me.pLoc.locv > 194 then
      --land on ground
      me.pOnGround = true
      me.pLoc.locv = 194
      me.pVel.locv = 0      
    end if
    
  end if  
  
  --Animate me
  me.ancestor.EnterFrame(aTime)
end

on SquashBugsBelow me
  
  if me.pVel.locv > 0 then
    --player is heading down
    
    vInRange = me.GetObjectsInRange(-10,10)
    
    repeat with vObj in vInRange
      if vObj.pType = #bug then
        vObj.pIsDead=true
      end if
    end repeat
    
  end if
end